--------------------------------------------------
-- Iir18b18b.vhd
-- by Toshio Iwata at digitalfilter.com 2009/05/06

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity Iir18b18b is
   Port (
          RST_N : In std_logic;
          MCLK : In std_logic;
          FSCLK : In std_logic;
          XIN : In std_logic_vector(17 downto 0);
          YOUT : Out std_logic_vector(17 downto 0);
          A0 : In std_logic_vector(17 downto 0);
          A1 : In std_logic_vector(17 downto 0);
          A2 : In std_logic_vector(17 downto 0);
          B1 : In std_logic_vector(17 downto 0);
          B2 : In std_logic_vector(17 downto 0) );
end Iir18b18b;

architecture RTL of Iir18b18b is
  signal X0, X1, X2 : signed(17 downto 0);
  signal Y1, Y2 : signed(17 downto 0);
  signal HnXn : signed(35 downto 0);
  signal HnXn_24b : signed(23 downto 0);
  signal XIN_sig : signed(17 downto 0);
  signal YOUT_sig : signed(25 downto 0);
  signal YOUT_round : signed(25 downto 0);
  signal ADDOUT : signed(25 downto 0);
  signal co10 : unsigned(3 downto 0);
  signal MuxXn : signed(17 downto 0);
  signal MuxHn : signed(17 downto 0);
  signal fsclk_dly1, fsclk_dly2 : std_logic;
  signal fsclk_edge : std_logic;

begin -- Iir18b18b
-------------------------------------------------------------------
gen_fsclk_dly : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then
    fsclk_dly2 <= '0';
    fsclk_dly1 <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    fsclk_dly2 <= fsclk_dly1;
    fsclk_dly1 <= FSCLK;
  end if;
end process gen_fsclk_dly;

-------------------------------------------------------------------
  fsclk_edge <= '1' when fsclk_dly2 = '0' and fsclk_dly1 = '1' 
              else '0';
              
-------------------------------------------------------------------
gen_co10 : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    co10 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( fsclk_edge = '1' ) then 
      co10 <= (others => '0');
    elsif( co10 /= 9 ) then  
      co10 <= co10 + 1;
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_MuxXn : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    MuxXn <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( co10 = 0 ) then 
      MuxXn <= X0;
    elsif( co10 = 1 ) then 
      MuxXn <= X1;
    elsif( co10 = 2 ) then 
      MuxXn <= X2;
    elsif( co10 = 3 ) then 
      MuxXn <= Y1;
    elsif( co10 = 4 ) then 
      MuxXn <= Y2;
    else 
      MuxXn <= (others => '0');
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_MuxHn : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    MuxHn <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( co10 = 0 ) then 
      for i in 17 downto 0 loop
        MuxHn(i) <= A0(i);
      end loop;
    elsif( co10 = 1 ) then 
      for i in 17 downto 0 loop
        MuxHn(i) <= A1(i);
      end loop;
    elsif( co10 = 2 ) then 
      for i in 17 downto 0 loop
        MuxHn(i) <= A2(i);
      end loop;
    elsif( co10 = 3 ) then 
      for i in 17 downto 0 loop
        MuxHn(i) <= B1(i);
      end loop;
    elsif( co10 = 4 ) then 
      for i in 17 downto 0 loop
        MuxHn(i) <= B2(i);
      end loop;
    else 
      MuxHn <= (others => '0');
    end if;
  end if;
end process;

-------------------------------------------------------------------
  HnXn <= MuxXn * MuxHn;

-------------------------------------------------------------------
gen_XIN_sig : process( XIN )
begin
  for i in 17 downto 0 loop
    XIN_sig(i) <= XIN(i);
  end loop;
end process;

----------------------------------------------------
gen_Xn : process( RST_N, MCLK )
begin 
  if( RST_N = '0' ) then 
    X2 <= (others => '0');
    X1 <= (others => '0');
    X0 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( fsclk_edge = '1' ) then 
      X2 <= X1;
      X1 <= X0;
      X0 <= XIN_sig;
    end if;
  end if;
end process;

----------------------------------------------------
gen_HnXn_24b: process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    HnXn_24b <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    for i in 23 downto 0 loop
      HnXn_24b(i) <= HnXn(i+11);
    end loop;
  end if;
end process;

----------------------------------------------------
  ADDOUT <= HnXn_24b + YOUT_sig;

----------------------------------------------------
gen_YOUT_sig : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    YOUT_sig <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( fsclk_edge = '1' ) then 
      YOUT_sig <= (others => '0');
    else 
      YOUT_sig <= ADDOUT;
    end if;
  end if;
end process;

----------------------------------------------------
  YOUT_round <= YOUT_sig + 16; -- to round it in bit 5

-------------------------------------------------------------------
-- ʂ̃}[WrbgŃI[ot[f
--rbg22`5Ƃ18rbgƂ(W1/2ɂȂĂ镪2{đE)
gen_Y1 : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then 
    Y1 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( co10 = 7 ) then 
      if( YOUT_round(25) = '0' and -- positive value
         ( YOUT_round(24) = '1' 
        or YOUT_round(23) = '1' 
        or YOUT_round(22) = '1' ) ) then -- overflow
        Y1 <= "011111111111111111";
      elsif( YOUT_round(25) = '1' and -- negative value
         ( YOUT_round(24) = '0' 
        or YOUT_round(23) = '0' 
        or YOUT_round(22) = '0' ) ) then -- underflow
        Y1 <= "100000000000000000";
      else 
        for i in 17 downto 0 loop
          Y1(i) <= YOUT_round(i+5); -- discard 5 bits (max 32)
        end loop;
      end if;
    end if;
  end if;
end process;

----------------------------------------------------
gen_Yn : process( RST_N, MCLK )
begin 
  if( RST_N = '0' ) then 
    Y2 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( co10 = 7 ) then 
      Y2 <= Y1;
    end if;
  end if;
end process;

----------------------------------------------------
gen_YOUT : process( Y1 )
begin
  for i in 17 downto 0 loop
    YOUT(i) <= Y1(i);
  end loop;
end process;

end; -- Iir18b18b
