--------------------------------------------------
-- IirSerial3.vhd
-- FIR Filter with 20bits data
-- by Toshio Iwata at digitalfilter.com 2009/05/06

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity IirSerial3 is
   Port (
          RST_N : In std_logic;
          MCLK : In std_logic;
          FSCLK : In std_logic;
          A0 : In std_logic_vector(17 downto 0);
          A1 : In std_logic_vector(17 downto 0);
          A2 : In std_logic_vector(17 downto 0);
          B1 : In std_logic_vector(17 downto 0);
          B2 : In std_logic_vector(17 downto 0);
          C0 : In std_logic_vector(17 downto 0);
          C1 : In std_logic_vector(17 downto 0);
          C2 : In std_logic_vector(17 downto 0);
          D1 : In std_logic_vector(17 downto 0);
          D2 : In std_logic_vector(17 downto 0);
          E0 : In std_logic_vector(17 downto 0);
          E1 : In std_logic_vector(17 downto 0);
          E2 : In std_logic_vector(17 downto 0);
          F1 : In std_logic_vector(17 downto 0);
          F2 : In std_logic_vector(17 downto 0);
          XIN : In std_logic_vector(15 downto 0);
          YOUT : Out std_logic_vector(15 downto 0) );
end IirSerial3;

architecture RTL of IirSerial3 is

component Iir18b18b
Port (
          RST_N : In std_logic;
          MCLK : In std_logic;
          FSCLK : In std_logic;
          XIN : In std_logic_vector(17 downto 0);
          YOUT : Out std_logic_vector(17 downto 0);
          A0 : In std_logic_vector(17 downto 0);
          A1 : In std_logic_vector(17 downto 0);
          A2 : In std_logic_vector(17 downto 0);
          B1 : In std_logic_vector(17 downto 0);
          B2 : In std_logic_vector(17 downto 0) );
end component;

  signal d_in_sig : std_logic_vector(17 downto 0);
  signal d_out_sig : std_logic_vector(17 downto 0);
  signal d_inter_sig : std_logic_vector(17 downto 0);
  signal d_inter2_sig : std_logic_vector(17 downto 0);

begin
-------------------------------------------------------------------
gen_d_in_sig : process(XIN) 
begin 
  if( XIN(15) = '0' ) then
    d_in_sig <= "00" & XIN;
  else
    d_in_sig <= "11" & XIN;
  end if;
end process;
 
--------------------------------------------------------------
gen_d_out_sig : process(d_out_sig) 
begin 
  if( d_out_sig(17) = '0' and
     (d_out_sig(16) = '1' or
      d_out_sig(15) = '1') ) then -- overflow
    YOUT <= (others => '1');
    YOUT(15) <= '0';
  elsif( d_out_sig(17) = '1' and
        (d_out_sig(16) = '0' or
         d_out_sig(15) = '0') ) then -- underflow
    YOUT <= (others => '0');
    YOUT(15) <= '1';
  else
    for i in 15 downto 0 loop
      YOUT(i) <= d_out_sig(i);
    end loop;
  end if; 
end process;

--------------------------------------------------------------
Iir18b18b_1 : Iir18b18b port map ( -- 1st iir filter
          RST_N => RST_N,
          MCLK => MCLK,
          FSCLK => FSCLK,
          XIN => d_in_sig,
          YOUT => d_inter_sig,
          A0 => a0,
          A1 => a1,
          A2 => a2,
          B1 => b1,
          B2 => b2 );

Iir18b18b_2 : Iir18b18b port map ( -- 2nd iir filter
          RST_N => RST_N,
          MCLK => MCLK,
          FSCLK => FSCLK,
          XIN => d_inter_sig,
          YOUT => d_inter2_sig,
          A0 => c0,
          A1 => c1,
          A2 => c2,
          B1 => d1,
          B2 => d2 );

Iir18b18b_3 : Iir18b18b port map ( -- 3rd iir filter
          RST_N => RST_N,
          MCLK => MCLK,
          FSCLK => FSCLK,
          XIN => d_inter2_sig,
          YOUT => d_out_sig,
          A0 => e0,
          A1 => e1,
          A2 => e2,
          B1 => f1,
          B2 => f2 );

end;
