-------------------------------
-- P_S.vhd
-- Parallel-Serial Converter
-- by Toshio Iwata at digitalfilter.com 2009/05/06

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity P_S is
   Port (
          RST_N : In std_logic; 
          MCLK : In std_logic; 
          LATCH_L : In std_logic; 
          LATCH_R : In std_logic; 
          BCLK : In std_logic; 
          LDATA : In std_logic_vector(15 downto 0); 
          RDATA : In std_logic_vector(15 downto 0); 
          SDATA : Out std_logic );
end P_S;

architecture STRUCTURE of P_S is

  signal rdata_tmp : std_logic_vector( 15 downto 0 );
  signal counter256 : unsigned( 7 downto 0 );

begin -- architecture P_S

-------------------------------------------------------------------
gen_rdata: process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    rdata_tmp <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( LATCH_L = '1' ) then
        rdata_tmp <= LDATA;
    elsif( LATCH_R = '1' ) then
        rdata_tmp <= RDATA;
    elsif( BCLK = '1' ) then
        rdata_tmp <= rdata_tmp(14 downto 0) & '0';
    end if;
  end if;
end process gen_rdata;

-------------------------------------------------------------------
gen_sdata: process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    SDATA <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    SDATA <= rdata_tmp(15);
  end if;
end process gen_sdata;

-------------------------------------------------------------------
gen_counter256 : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    counter256 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( LATCH_R = '1' ) then 
      counter256 <= (others => '0'); 
    elsif( BCLK = '1' ) then 
      counter256 <= counter256 + 1;
    end if;
  end if;
end process gen_counter256;

end; -- architecture P_S
