-------------------------------
-- S_P.vhd
-- Serial-Parallel Converter
-- by Toshio Iwata at digitalfilter.com 2009/05/06

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity S_P is
   Port (
          RST_N : In std_logic; 
          MCLK : In std_logic; 
          SDATA : In std_logic;
          LRCLK : In std_logic; 
          BCLK : In std_logic; 
          LDATA : Out std_logic_vector(15 downto 0); 
          RDATA : Out std_logic_vector(15 downto 0) ); 
end S_P;

architecture STRUCTURE of S_P is

  signal pdata_tmp : std_logic_vector( 15 downto 0 );
  signal rdata_latch : std_logic;
  signal ldata_latch : std_logic;
  signal counter256 : unsigned( 7 downto 0 );

begin -- architecture S_P

-------------------------------------------------------------------
gen_pdata_tmp : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    pdata_tmp <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( BCLK = '1' ) then 
      pdata_tmp <= pdata_tmp(14 downto 0) & SDATA;
    end if;
  end if;
end process gen_pdata_tmp;

-------------------------------------------------------------------
gen_ldata : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    ldata <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( ldata_latch = '1' ) then
      LDATA <= pdata_tmp;
    end if;
  end if;
end process gen_ldata;

-------------------------------------------------------------------
gen_rdata : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    RDATA <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( rdata_latch = '1' ) then
      RDATA <= pdata_tmp;
    end if;
  end if;
end process gen_rdata;

-------------------------------------------------------------------
gen_counter256 : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    counter256 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( LRCLK = '1' ) then 
      counter256 <= (others => '0'); 
    elsif( BCLK = '1' ) then 
      counter256 <= counter256 + 1;
    end if;
  end if;
end process gen_counter256;

-------------------------------------------------------------------
  ldata_latch <= '1' when counter256 = 16 else '0';
  rdata_latch <= '1' when counter256 = 48 else '0';

end; -- architecture S_P
