-------------------------------
-- initial_latch.vhd
-- Initial DF/F to latch serial data
-- by Toshio Iwata at digitalfilter.com 2009/05/06

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity initial_latch is
   Port (
          RST_N : In std_logic;
          MCLK : In std_logic;
          ADATA : In std_logic;
          ADATAO : Out std_logic;
          ABCLK : In std_logic;
          ABCLKO : Out std_logic;
          ALRCLK : In std_logic;
          ALRCLKO : Out std_logic;
          ALRCLKO2 : Out std_logic );
end initial_latch;

architecture STRUCTURE of initial_latch is

  signal ADATA_dly0 : std_logic;
  signal ADATA_dly1 : std_logic;
  signal ALRCLK_dly0 : std_logic;
  signal ALRCLK_dly1 : std_logic;
  signal ABCLK_dly0 : std_logic;
  signal ABCLK_dly1 : std_logic;

begin -- architecture initial_latch

-------------------------------------------------------------------
gen_ADATA_dly : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    ADATA_dly1 <= '0';
    ADATA_dly0 <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    ADATA_dly1 <= ADATA_dly0;
    ADATA_dly0 <= ADATA;
  end if;
end process;

-------------------------------------------------------------------
gen_ALRCLK_dly : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    ALRCLK_dly1 <= '0';
    ALRCLK_dly0 <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    ALRCLK_dly1 <= ALRCLK_dly0;
    ALRCLK_dly0 <= ALRCLK;
  end if;
end process;

-------------------------------------------------------------------
gen_ABCLK_dly : process( RST_N, MCLK ) 
begin
  if(RST_N = '0') then
    ABCLK_dly1 <= '0';
    ABCLK_dly0 <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    ABCLK_dly1 <= ABCLK_dly0;
    ABCLK_dly0 <= ABCLK;
  end if;
end process;

-------------------------------------------------------------------
  ADATAO <= ADATA_dly0;
  ALRCLKO <= ALRCLK_dly0 and not ALRCLK_dly1;
  ABCLKO <= ABCLK_dly0 and not ABCLK_dly1;

  ALRCLKO2 <= not ALRCLK_dly0 and ALRCLK_dly1;

end; -- architecture initial_latch
